libname dur "C:\Users\EDUGUET\FacOuvrages\Ouvrage_Duree\data";

/* VERSION OUVRAGE */

/* Introduction  SAS IML : estimation du paramtre de la loi exponentielle */
/* de sa moyenne, de sa mdiane des carts types correspondants             */
/* sorties graphiques */

options linesize=64;

data duree; set dur.base_duree;
presence=1-censure;
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let outp=_outp;
%let outi=_outi;
%let tmax=30;

proc iml;
/* lecture des donnes et stat de base */
start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd;
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;
/* estimation des paramtres */
start estim;
print "Exponential distribution",, "S(t)=exp(-h.t)",,;
/* no optimization needed */
/* distribution parameter */
   h=nd/sum(yi);  vh=h##2/nd; sh=sqrt(vh); th=abs(h)/sh; ph=1-probchi(th##2,1);
/* mean duration */
   mt=1/h; vmt=vh/h##4; smt=sqrt(vmt); tmt=abs(mt)/smt; pmt=1-probchi(tmt##2,1);
/* median duration */
   md=log(2)/h; vmd=log(2)##2#vh/h##4; smd=sqrt(vmd); tmd=abs(md)/smd; pmd=1-probchi(tmd##2,1);
/* log-likelihood */
   lv=sum(di#log(h)-h#yi);
/* 1st output matrix */
   resp=(lv||.||.||.)//(h||sh||th||ph)//(md||smd||tmd||pmd)//(mt||smt||tmt||pmt);
   labc="Parameter"||"Std_Dev"||"Student"||"P_Value";
   labr="Log-likelihood"//"h"//"Median"//"Mean";
   print resp[colname=labc rowname=labr];
finish estim;
/* prvisions */
start prev;
tmin=min(yi);
tmax=&tmax;
if tmax=0 then tmax=max(yi);
tstep=1;
do time=tmin to tmax by tstep;
   st=exp(-h#time);
   ht=h;
   ct=1-st;
   ft=h#exp(-h#time);
   resi=resi//(time||st||ht||ct||ft);
end;
   resi=(0||1||h||0||h)//resi;
   labi=nom_y||"SDF"||"Hasard"||"CDF"||"Density";
finish prev;
/* sorties */
start sortie;
  create &outp from resp[colname=labc]; append from resp;
  create _np from labr[colname="Statistic"]; append from labr;
  create &outi from resi[colname=labi]; append from resi;
finish sortie;

/* lancement du programme */
run lecture;
run estim;
run prev;
run sortie;

quit;
run;
data &outp; merge _np &outp;
run;

/* graphique 1 */
data tab1; set _outi;
keep duree_emp sdf Fonction;
Fonction="S(t)";
rename sdf=f;
run;

data tab2; set _outi;
keep duree_emp cdf Fonction;
Fonction="F(t)";
rename cdf=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;

/* graphique 2 */

data tab1; set _outi;
keep duree_emp hasard Fonction;
Fonction="h(t)";
rename hasard=f;
run;

data tab2; set _outi;
keep duree_emp density Fonction;
Fonction="f(t)";
rename density=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;
