libname dur "C:\Users\EDUGUET\FacOuvrages\Ouvrage_Duree\data";

/* VERSION OUVRAGE */


/* estimation des paramtres de la loi Gamma  1 paramtre */

options linesize=64;

data duree; set dur.base_duree;
presence=1-censure;
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let outp=_outp;
%let outi=_outi;
%let tmax=30;

proc iml;

start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd;
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

start igf(beta,x);
f=probchi(2#x,2#beta);
do i=1 to nrow(f);
f[i]=min(f[i],1-1e-6);
end;
return (f);
finish;

start lv(theta) global(di,yi,lyi);
beta=exp(theta[1]);
lfi=(beta-1)#lyi-yi-lgamma(beta);
lsi=log(1-igf(beta,yi));
lv=sum(di#lfi+(1-di)#lsi);
return (lv);
finish;

start estim;
print "One parameter Gamma distribution",, "S(t)=1-g(beta,t)",
"where g(,) is the regularized incomplete Gamma function",;
lyi=log(yi);
beta0=sum(yi)/nd;
theta0=log(beta0);
opt=j(11,1,0);
opt[1]=1; opt[2]=5;
call nlpnra(rc,theta,"lv",theta0,opt);
print "Result code (must be >0) =" rc;
call nlpfdd(lv1,gr,he,"lv",theta);
elas=abs(theta#gr/lv1);
print "Log-likelihood at maximum =" lv1,
      "Gradient" gr,
      "Elasticity =" elas,
      "Second order derivative =" he ;
vtheta=-1/he; 
beta=exp(theta); 
vb=beta##2#vtheta;
sb=sqrt(vecdiag(vb));
tb=beta/sb;
pb=1-probchi(tb##2,1);
resp=(lv1||.||.||.)//(beta||sb||tb||pb);
labc="Estimation"||"Std_Dev"||"Student"||"Pvalue";
labr="Log-likelihood"//"Beta"//"Median";
me=cinv(0.5,2#beta)/2;
resp=resp//(me||.||.||.);
print resp[colname=labc rowname=labr];
finish estim;

start prev;
tmin=min(yi);
tmax=&tmax; if tmax<=0 then tmax=max(yi);
tstep=1;
do time=tmin to tmax by tstep;
fi=time##(beta-1)#exp(-time)/gamma(beta);
si=1-igf(beta,time);
hi=fi/si;
ci=1-si;
resi=resi//(time||fi||si||hi||ci);
end;
labi="&y"||"Density"||"SDF"||"Hasard"||"CDF";
finish prev;

start sortie;
  create &outp from resp[colname=labc]; append from resp;
  create _np from labr[colname="Statistic"]; append from labr;
  create &outi from resi[colname=labi]; append from resi;
finish sortie;

run lecture;
run estim;
run prev;
run sortie;

quit;
run;


data &outp; merge _np &outp;
run;

/* graphique 1 */
data tab1; set _outi;
keep duree_emp sdf Fonction;
Fonction="S(t)";
rename sdf=f;
run;

data tab2; set _outi;
keep duree_emp cdf Fonction;
Fonction="F(t)";
rename cdf=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;

/* graphique 2 */

data tab1; set _outi;
keep duree_emp hasard Fonction;
Fonction="h(t)";
rename hasard=f;
run;

data tab2; set _outi;
keep duree_emp density Fonction;
Fonction="f(t)";
rename density=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;
