libname dur "C:\Users\EDUGUET\FacOuvrages\Ouvrage_Duree\data";

/* VERSION OUVRAGE */


/* estimation des paramtres de la loi de Lomax Gamma  1 paramtre */

options linesize=64;

data duree; set dur.base_duree;
presence=1-censure;
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let outp=_outp;
%let outi=_outi;
%let tmax=30;


proc iml;
start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd; 
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

start lv(theta) global(di,yi);
h=exp(theta);
lsi=-log(1+h#yi);
lv=sum(di#(theta+lsi)+lsi);
return (lv);
finish;

start sc(theta) global(di,yi,nd); 
h=exp(theta);
der=nd-sum((1+di)#h#yi/(1+h#yi));
return (der);
finish;

start estim;
print "One parameter Lomax distribution",, "S(t)=1/(1+h.t)",;
lyi=log(yi);
h0=sum(yi)/nd;
theta0=log(h0);
opt=j(11,1,0);
opt[1]=1; opt[2]=1;
call nlpnra(rc,theta,"lv",theta0,opt) grd="sc";
print "Result code (must be >0) =" rc;
call nlpfdd(lv1,gr,he,"lv",theta) grd="sc";
elas=abs(theta#gr/lv1);
print "Log-likelihood at maximum =" lv1,
      "Gradient" gr,
      "Elasticity =" elas,
      "Second order derivative =" he ;
vtheta=-1/he; 
h=exp(theta); 
vb=h##2#vtheta;
sb=sqrt(vecdiag(vb));
tb=h/sb;
pb=1-probchi(tb##2,1);
resp=h||sb||tb||pb;
me=1/h; dg=-1/h##2; sme=sqrt(dg##2#vb); tme=abs(me)/sme; pme=1-probchi(tme##2,1);
resp=(lv1||.||.||.)//resp//(me||sme||tme||pme);
labc="Parameter"||"Std_Dev"||"Student"||"Pvalue";
labr="Log-likelihood"//"h"//"Median";
print "This distribution has NO mean",,resp[colname=labc rowname=labr];
finish estim;

start prev;
tmin=min(yi); 
tmax=&tmax; if tmax<=0 then tmax=max(yi); 
tstep=1;
do time=tmin to tmax by tstep;
si=1/(1+h#time);
hi=h/(1+h#time);
fi=si#hi;
ci=1-si;
resi=resi//(time||fi||si||hi||ci);
end;
resi=(0||h||1||h||0)//resi;
labi="&y"||"Density"||"SDF"||"Hasard"||"CDF";
finish prev;

start sortie;
  create &outp from resp[colname=labc]; append from resp;
  create _np from labr[colname="Statistic"]; append from labr;
  create &outi from resi[colname=labi]; append from resi;
finish sortie;

/* lancement du programme */
run lecture;
run estim;
run prev;
run sortie;

quit;

data &outp; merge _np &outp;
run;

/* graphique 1 */
data tab1; set _outi;
keep duree_emp sdf Fonction;
Fonction="S(t)";
rename sdf=f;
run;

data tab2; set _outi;
keep duree_emp cdf Fonction;
Fonction="F(t)";
rename cdf=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;

/* graphique 2 */

data tab1; set _outi;
keep duree_emp hasard Fonction;
Fonction="h(t)";
rename hasard=f;
run;

data tab2; set _outi;
keep duree_emp density Fonction;
Fonction="f(t)";
rename density=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;


