libname dur "C:\Users\EDUGUET\FacOuvrages\Ouvrage_Duree\data";

/* VERSION OUVRAGE */


/* estimation des paramtres de la loi de Weibull */

options linesize=64;

data duree; set dur.base_duree;
presence=1-censure;
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let outp=_outp;
%let outi=_outi;
%let tmax=30;

proc iml;
start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd;
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;
/* log-likelihood */
start lv(theta) global(di,yi,lyi,nd);
t1=theta[1]; t2=theta[2]; alpha=exp(t1); h=exp(t2);
lv=nd#(t1+t2)+sum((alpha-1)#di#lyi-h#yi##alpha);
return (lv);
finish;
start sc(theta) global(di,yi,lyi,nd);
t1=theta[1]; t2=theta[2]; alpha=exp(t1); h=exp(t2);
yia=yi##alpha;
der1=nd+alpha#sum(lyi#(di-h#yia));
der2=nd-h#sum(yia);
der=(der1||der2);
return (der);
finish;
start hes(theta) global(di,yi,lyi);
t1=theta[1]; t2=theta[2]; alpha=exp(t1); h=exp(t2);
yia=yi##alpha;
he11=alpha#sum(lyi#(di-(alpha#lyi+1)#h#yia));
he12=-alpha#h#sum(lyi#yia);
he22=-h#sum(yia);
he=(he11||he12)//(he12||he22);
return (he);
finish;

start estim;
print "Weibull distribution",, "S(t)=exp(-h.t^alpha)",,;
lyi=log(yi);
h0=nd/sum(yi); alpha0=1;
theta0=log(alpha0||h0); /* transformed parameters */
opt=j(11,1,0);
opt[1]=1; opt[2]=1;
/*d1=sc(theta0);
call nlpfdd(f,d2,h,"lv",theta0);
print d1, d2;
h1=hes(theta0);
call nlpfdd(f,d2,h2,"lv",theta0) grd="sc";
print h1, h2;*/
call nlpnra(rc,theta,"lv",theta0,opt) grd="sc" hes="hes";
print "Result code (must be >0) =" rc;
lv1=lv(theta); gr=t(sc(theta)); he=hes(theta);
theta=t(theta); elas=abs(theta#gr/lv1); vp=eigval(he);
print "Log-likelihood at maximum =" lv1,
      "Gradient" gr,
      "Elasticities =" elas,
      "Hessian eigenvalues =" vp ;
vtheta=inv(-he); 
b=exp(theta);
alpha=b[1]; h=b[2];
dg=(alpha||0)//(0||h);
vb=t(dg)*vtheta*dg;
sb=sqrt(vecdiag(vb));
tb=b/sb;
pb=1-probchi(tb##2,1);
resp=b||sb||tb||pb;
mt=gamma(1+alpha)#h##(-1/alpha);
dmt=mt#(digamma(1+alpha)+log(h)/alpha##2) // mt#(-1/alpha/h);
vmt=t(dmt)*vb*dmt; smt=sqrt(vmt); tmt=mt/smt; pmt=1-probchi(tmt##2,1);
md=(log(2)/h)##(1/alpha);
dmd=-md#(log(log(2)/h)/alpha##2) // (-md/h/alpha);
vmd=t(dmd)*vb*dmd; smd=sqrt(vmd); tmd=md/smd; pmd=1-probchi(tmd##2,1);
resp=(lv1||.||.||.)//resp//(md||smd||tmd||pmd)//(mt||smt||tmt||pmt);
labc="Parameter"||"Std_Dev"||"Student"||"P_Value";
labr="Log-likelihood"//"alpha"//"h"//"Median"//"Mean";
print resp[colname=labc rowname=labr];
test=abs(alpha-1)/sb[1]; ptest=1-probchi(test##2,1);
print "Test H0 : alpha=1 (exponential distribution)",
       "Student =" test,
       "p value =" ptest;
finish estim;

start prev;
tmin=min(yi);
tmax=&tmax; if tmax<=0 then tmax=max(yi); tby=1;
do time=tmin to tmax by tby;
 st=exp(-h#time##alpha);
 ht=h#alpha#time##(alpha-1);
 ct=1-st;
 ft=ht#st;
 resi=resi//(time||st||ht||ct||ft);
end;
if alpha>=1 then do; do time=1 to 0 by -0.1; 
                     st=exp(-h#time##alpha);
                     ht=h#alpha#time##(alpha-1);
                     ct=1-st;
                     ft=ht#st;
                     resi=(time||st||ht||ct||ft)//resi;
					 end;
					 end;
labi=nom_y||"SDF"||"Hasard"||"CDF"||"Density";
finish prev;

/* output */
start sortie;
create &outp from resp[colname=labc]; append from resp;
create _np from labr[colname="Estimate"]; append from labr;
create &outi from resi[colname=labi]; append from resi;
finish sortie;

   run lecture;
   run estim;
   run prev;
   run sortie;
   quit;
  data &outp; merge _np &outp;
  run;
/* graphique 1 */
data tab1; set _outi;
keep duree_emp sdf Fonction;
Fonction="S(t)";
rename sdf=f;
run;

data tab2; set _outi;
keep duree_emp cdf Fonction;
Fonction="F(t)";
rename cdf=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;

/* graphique 2 */

data tab1; set _outi;
keep duree_emp hasard Fonction;
Fonction="h(t)";
rename hasard=f;
run;

data tab2; set _outi;
keep duree_emp density Fonction;
Fonction="f(t)";
rename density=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;


