libname dur "C:\Users\EDUGUET\FacOuvrages\Ouvrage_Duree\data";

/* VERSION OUVRAGE */


/* estimation des paramtres de la loi Gamma 2 */

options linesize=64;

data duree; set dur.base_duree;
presence=1-censure;
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let outp=_outp;
%let outi=_outi;
%let tmax=30;

%let i=100;
%let e=1e-6;
%let c=10;


proc iml;
start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd;
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

start igf(beta,x) global(epsilon);
f=probchi(2#x,2#beta);
do i=1 to nrow(x);
f[i]=min(f[i],1-epsilon);
end;
return (f);
finish;

start lv(theta) global(di,yi,lyi);
t1=theta[1]; t2=theta[2];
beta=exp(t1); h=exp(t2);
lfi=beta#t2+(beta-1)#lyi-h#yi-lgamma(beta);
lsi=log(1-igf(beta,h#yi));
lv=sum(di#(lfi-lsi)+lsi);
return (lv);
finish;

start estim;
print "Two-parameter Gamma distribution",, "S(t)=1-g(beta,h#t)",
"where g(,) is the regularized incomplete Gamma function",;
epsilon=1e-6;
lyi=log(yi);
my=sum(yi)/nd;
theta0=log(nd/n)||log(1/my);
lv0=lv(theta0);
print "Starting parameter =" theta0,
      "Starting log-likelihood =" lv0,,;
maxc=&c; maxiter=&i; maxe=&e;
labopt="Iteration"||"Log-likelihood"||"Max Abs Elasticity"||"Sub-Iteration";
do iter=1 to maxiter;
 call nlpfdd(lv1,sc,he,"lv",theta0);
 lambda=1;
 step=sc*inv(he);
 theta=theta0-lambda#step;
 vp=max(eigval(he));
 print "vp =" vp;
 lv1=lv(theta); 
 if lv1<lv0 then do;
             do c=1 to maxc;
			 lambda=lambda/2;
	         theta=theta0-lambda#step;
		     lv1=lv(theta);
             if lv1>=lv0 then do; print lambda; c=maxc; end;
			 end;
		   end;
elas=max(abs(theta#sc/lv1));
riter=(iter||lv1||elas||lambda);
print riter[colname=labopt];
theta0=theta; lv0=lv1;
if elas<&e then iter=maxiter;
end;
/*b=exp(theta1); print b , sc, elas;
opt=j(11,1,0);
opt[1]=1; opt[2]=1;
theta0=log(nd/n)||log(1/my);
call nlpnra(rc,theta,"lv",theta0,opt);
b2=exp(theta); print b2;*/
call nlpfdd(lv1,gr,he,"lv",theta);
gr=t(gr); theta=t(theta); elas=abs(theta#gr/lv1); vp=eigval(he);
print "Log-likelihood at maximum =" lv1,
      "Gradient" gr,
      "Elasticities =" elas,
      "Hessian eigenvalues =" vp ;
vtheta=inv(-he); 
b=exp(theta); beta=b[1]; h=b[2];
db=(beta||0)//(0||h);
vb=db*vtheta*t(db);
sb=sqrt(vecdiag(vb));
tb=b/sb;
pb=1-probchi(tb##2,1);
resp=b||sb||tb||pb;
labc="Params"||"Std_Dev"||"Student"||"Pvalue";
labr="Log-likelihood"//"Beta"//"H"//"Mean"//"Median";
md=cinv(0.5,2#beta)/(2#h);
mt=1/h; smt=sqrt((1/h)##2#sb[2]); tmt=mt/smt; pmt=1-probchi(tmt##2,1);
resp=(lv1||.||.||.)//resp//(mt||smt||tmt||pmt)//(md||.||.||.);
print resp[colname=labc rowname=labr];
stat_exp=abs(beta-1)/sb[1]; pval_exp=1-probchi(stat_exp##2,1);
stat_gam=abs(h-1)/sb[2]; pval_gam=1-probchi(stat_gam##2,1);
print "Test H0 : beta=1 (loi exponentielle) " stat_exp, "p-value =" pval_exp,,,
      "Test H0 : h=1 (loi Gamma  1 paramtre)" stat_gam, "p-value" pval_gam;
finish estim;
start prev;
tmin=min(yi); 
tmax=&tmax; if tmax<=0 then tmax=max(yi); 
tby=1;
do time=tmin to tmax by tby;
fi=h##beta#time##(beta-1)#exp(-h#time)/gamma(beta);
si=1-igf(beta,h#time);
hi=fi/si;
ci=1-si;
resi=resi//(time||fi||si||hi||ci);
end;
if beta>=1 then do;
           do time=0.9 to 0 by -0.1;
           fi=h##beta#time##(beta-1)#exp(-h#time)/gamma(beta);
           si=1-igf(beta,h#time);
           hi=fi/si;
           ci=1-si;
           resi=(time||fi||si||hi||ci)//resi;
		   end;
		   end;
labi="&y"||"Density"||"SDF"||"Hasard"||"CDF";
create &outi from resi[colname=labi]; append from resi;
finish prev;
start sortie;
create &outp from resp[colname=labc]; append from resp;
create _np from labr[colname="Estimate"]; append from labr;
create &outi from resi[colname=labi]; append from resi;
finish sortie;

run lecture;
run estim;
run prev;
run sortie;

quit;

data &outp; merge _np &outp;
run;

/* graphique 1 */
data tab1; set _outi;
keep duree_emp sdf Fonction;
Fonction="S(t)";
rename sdf=f;
run;

data tab2; set _outi;
keep duree_emp cdf Fonction;
Fonction="F(t)";
rename cdf=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;

/* graphique 2 */

data tab1; set _outi;
keep duree_emp hasard Fonction;
Fonction="h(t)";
rename hasard=f;
run;

data tab2; set _outi;
keep duree_emp density Fonction;
Fonction="f(t)";
rename density=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;


