
/* estimation des paramtres de la loi log-normale */
libname dur "C:\Users\EDUGUET\FacOuvrages\Ouvrage_Duree\data";

/* VERSION OUVRAGE */


options linesize=64;

data duree; set dur.base_duree;
presence=1-censure;
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let tmax=30;
%let i=100;
%let e=1e-8;
%let c=10;
%let outp=_outp;
%let outi=_outi;


proc iml;
start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd;
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

start cphi(x) global(epsilon);
f=1-probnorm(x);
do i=1 to nrow(x);
f[i]=max(f[i],epsilon);
end;
return (f);
finish;

start lv(theta) global(di,lyi,epsilon);
t1=theta[1];
alpha=exp(t1); mu=theta[2];
zi=alpha#lyi-mu;
lfi=t1-lyi+log(pdf("normal",zi,0,1));
lsi=log(cphi(zi));
lv=sum(di#lfi+(1-di)#lsi);
return (lv);
finish;

start sc(theta) global(di,lyi,epsilon,nd);
alpha=exp(theta[1]); mu=theta[2];
zi=alpha#lyi-mu;
wi=di#zi+(1-di)#pdf("normal",zi,0,1)/cphi(zi);
der1=nd-alpha#sum(lyi#wi);
der2=sum(wi);
der=(der1||der2);
return (der);
finish;

start estim;
print "Log-normal distribution",, "S(t)=1-Phi((ln(t)-m)/sigma)",,
"where Phi() is the CDF of the standard normal distribution",;
epsilon=1e-6;
lyi=log(yi);
mly=sum(lyi)/n; sly=sqrt(ssq(lyi)/n-mly##2);
theta0=(log(1/sly)||mly/sly);
lv0=lv(theta0);

/*d1=sc(theta0);
call nlpfdd(f,d2,h,"lv",theta0);
print d1, d2;
finish estim;
run lecture; run estim;
quit;
run;*/
print "Starting parameter =" theta0,
      "Starting log-likelihood =" lv0,,;
maxc=&c; maxiter=&i; maxe=&e;
labopt="Iteration"||"Log-likelihood"||"Max Abs Elasticity"||"Sub-Iteration";
do iter=1 to maxiter;
 call nlpfdd(lv1,sc,he,"lv",theta0) grd="sc";
 lambda=1;
 step=sc*inv(he);
 theta=theta0-lambda#step;
 vp=max(eigval(he));
 print "vp =" vp;
 lv1=lv(theta); 
 if lv1<lv0 then do;
             do c=1 to maxc;
			 lambda=lambda/2;
	         theta=theta0-lambda#step;
		     lv1=lv(theta);
             if lv1>=lv0 then c=maxc; 
			 end;
		   end;
elas=max(abs(theta#sc/lv1));
riter=(iter||lv1||elas||lambda);
print riter[colname=labopt];
theta0=theta; lv0=lv1;
if elas<&e then iter=maxiter;
end;
/*b=exp(-theta[1])||theta[2]#exp(-theta[1]); 
opt=j(11,1,0);
opt[1]=1; opt[2]=1;
theta0=(log(1/sly)||mly);
call nlpnra(rc,theta,"lv",theta0,opt);
b2=exp(-theta[1])||theta[2]#exp(-theta[1]); print b2;
finish estim; run lecture ; run estim; quit; run;*/
call nlpfdd(lv1,gr,he,"lv",theta) grd="sc";
gr=t(gr); theta=t(theta); elas=abs(theta#gr/lv1); vp=eigval(he);
print "Log-likelihood at maximum =" lv1,
      "Gradient =" gr,
      "Elasticities =" elas,
      "Hessian eigenvalues =" vp ;
vtheta=inv(-he); 
t1=theta[1]; t2=theta[2];
b=exp(-t1)//t2#exp(-t1);
sigma=b[1]; m=b[2];
/* FAIRE LES DERIVEES */
db=(-exp(-t1)||0)//(-t2#exp(-t1)||exp(-t1));
vb=db*vtheta*t(db);
sb=sqrt(vecdiag(vb));
tb=b/sb;
pb=1-probchi(tb##2,1);
resp=b||sb||tb||pb;
labc="Params"||"Std_Dev"||"Student"||"Pvalue";
labr="Log-likelihood"//"Sigma"//"m"//"Mean"//"Median";
md=exp(m); smd=sqrt(md##2#vb[2,2]); tmd=md/smd; pmd=1-probchi(tmd##2,1);
mt=exp(m+sigma##2/2); dmt=(sigma#mt||mt);
smt=sqrt(dmt*vb*t(dmt)); tmt=mt/smt; pmt=1-probchi(tmt##2,1);
resp=(lv1||.||.||.)//resp//(mt||smt||tmt||pmt)//(md||smd||tmd||pmd);
/* recherche du hasard maximum */
start has(x) global(alpha,mu);
z=alpha#x-mu;
f=pdf("normal",z,0,1)#alpha/exp(x);
s=cphi(z);
has=log(f)-log(s);
return (has);
finish;
opt=j(11,1,0); opt[1]=1; opt[2]=0; alpha=exp(t1); mu=t2; lt0=log(md);
call nlpnra(rc,lt1,"has",lt0,opt); /* on prend la mdiane comme valeur initiale */
tmax=exp(lt1); hmax=exp(has(lt1));
print "The hasard function reaches a maximum at T =" tmax,
      "Its maximum value is h =" hmax;
print resp[colname=labc rowname=labr];
finish estim;
start prev;
tmin=0.1/*min(yi)*/; 
tmax=&tmax; if tmax<=0 then tmax=max(yi); tby=0.1;
alpha=exp(t1); mu=t2;
do time=tmin to tmax by tby;
zi=alpha#log(time)-mu;
fi=pdf("normal",zi,0,1)#alpha/time;
si=cphi(zi);
hi=fi/si;
ci=1-si;
resi=resi//(time||fi||si||hi||ci);
end;

labi="&y"||"Density"||"SDF"||"Hasard"||"CDF";
finish prev;

start sortie;
create &outp from resp[colname=labc]; append from resp;
create _np from labr[colname="Estimate"]; append from labr;
create &outi from resi[colname=labi]; append from resi;
finish sortie;

run lecture;
run estim;
run prev;
run sortie;

quit;

data &outp; merge _np &outp;
run;



/* graphique 1 */
data tab1; set _outi;
keep duree_emp sdf Fonction;
Fonction="S(t)";
rename sdf=f;
run;

data tab2; set _outi;
keep duree_emp cdf Fonction;
Fonction="F(t)";
rename cdf=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;

/* graphique 2 */

data tab1; set _outi;
keep duree_emp hasard Fonction;
Fonction="h(t)";
rename hasard=f;
run;

data tab2; set _outi;
keep duree_emp density Fonction;
Fonction="f(t)";
rename density=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;


