
/* estimation des paramtres de la loi log-normale */
libname dur "C:\Users\EDUGUET\FacOuvrages\Ouvrage_Duree\data";

/* VERSION OUVRAGE */


options linesize=64;

data duree; set dur.base_duree;
presence=1-censure;
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let tmax=30;

/* estimation des paramtres de la loi log-logistique */

%let i=100;
%let e=1e-8;
%let c=10;
%let outp=_outp;
%let outi=_outi;


proc iml;
start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd;
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

start lv(theta) global(di,lyi,epsilon,yi,nd);
t1=theta[1]; t2=theta[2];
alpha=exp(t1); h=exp(t2);
lv=nd#(t1+t2)+sum((alpha-1)#di#lyi-(1+di)#log(1+h#yi##alpha));
return (lv);
finish;

start sc(theta) global(di,lyi,epsilon,yi,nd);
t1=theta[1]; t2=theta[2];
alpha=exp(t1); h=exp(t2);
zi1=h#yi##alpha;
zi=zi1/(1+zi1);
der1=nd+alpha#sum(lyi#(di-(1+di)#zi));
der2=nd-sum((1+di)#zi);
der=(der1||der2);
return (der);
finish;

start hes(theta) global(di,lyi,epsilon,yi,nd);
t1=theta[1]; t2=theta[2];
alpha=exp(t1); h=exp(t2);
zi1=h#yi##alpha;
zi=zi1/(1+zi1);
zi2=zi#(1-zi);
h11=alpha#sum(lyi#(di-(1+di)#zi-alpha#(1+di)#zi2#lyi));
h12=-alpha#sum((1+di)#zi2#lyi);
h22=-sum((1+di)#zi2);
he=(h11||h12)//(h12||h22);
return (he);
finish;

start estim;
print "Log-logistic",, "S(t)=1/(1+h.t^alpha)",,
"we also give S(t)=1-F((ln(t)-m)/delta)",
"where F is the CDF of the (0,1) Logistic distribution",
"and sigma=delta.pi/sqrt(3) for comparison with the log-normal distribution",;
epsilon=1e-6;
lyi=log(yi);
pi=arcos(-1);
mly=sum(lyi)/n; sly=sqrt(ssq(lyi)/n-mly##2);
alpha0=pi/sly/sqrt(3); h0=exp(-alpha0#mly);
theta0=(log(alpha0)||log(h0));
lv0=lv(theta0);
/*
d1=sc(theta0); h1=hes(theta0);
call nlpfdd(f,d2,h2,"lv",theta0);
print d1, d2; print h1, h2;
finish estim; run lecture; run estim; quit; run;*/
print "Starting parameter =" theta0,
      "Starting log-likelihood =" lv0,,;
maxc=&c; maxiter=&i; maxe=&e;
labopt="Iteration"||"Log-likelihood"||"Max Abs Elasticity"||"Sub-Iteration";
theta=theta0;
do iter=1 to maxiter;
 lambda=1;
 sc=sc(theta); he=hes(theta);
 step=sc*inv(he);
 theta=theta0-lambda#step;
 vp=max(eigval(he));
 print "vp =" vp;
 lv1=lv(theta); 
 if lv1<lv0 then do;
             do c=1 to maxc;
			 lambda=lambda/2;
	         theta=theta0-lambda#step;
		     lv1=lv(theta);
             if lv1>=lv0 then c=maxc; 
			 end;
		   end;
elas=max(abs(theta#sc/lv1));
riter=(iter||lv1||elas||lambda);
print riter[colname=labopt];
theta0=theta; lv0=lv1;
if elas<&e then iter=maxiter;
end;
/*b1=exp(theta[1])||exp(theta[2]); print b1;
opt=j(11,1,0);
opt[1]=1; opt[2]=1;
alpha0=pi/sly/sqrt(3); h0=exp(-alpha0#mly);
theta0=(log(alpha0)||log(h0));
call nlpnra(rc,theta,"lv",theta0,opt);
b2=exp(theta[1])||exp(theta[2]); print b2;
finish estim; run lecture ; run estim; quit; run;*/
lv1=lv(theta); gr=t(sc(theta)); he=hes(theta);
theta=t(theta); elas=abs(theta#gr/lv1); vp=eigval(he);
print "Log-likelihood at maximum =" lv1,
      "Gradient =" gr,
      "Elasticities =" elas,
      "Hessian eigenvalues =" vp ;
vtheta=inv(-he); 
t1=theta[1]; t2=theta[2];
b=exp(theta);
alpha=b[1]; h=b[2];
db=(alpha||0)//(0||h);
vb=db*vtheta*t(db);
sb=sqrt(vecdiag(vb));
tb=b/sb;
pb=1-probchi(tb##2,1);
b1=(1/alpha)//(-log(h)/alpha);
db1=( (-1/alpha##2)||0 )//( (-log(h)/alpha##2)|| (-1/alpha/h) );
vb1=db1*vb*t(db1);
sb1=sqrt(vecdiag(vb1));
tb1=b1/sb1;
pb1=1-probchi(tb1##2,1);
sigma=pi/alpha/sqrt(3);
dsig=-pi/alpha##2/sqrt(3);
vsig=dsig##2#vb[1,1];
ssig=sqrt(vsig);
tsig=sigma/ssig;
psig=1-probchi(tsig##2,1);
resp=(lv1||.||.||.)//(b||sb||tb||pb)//(b1||sb1||tb1||pb1)//(sigma||ssig||tsig||psig);
if alpha>1 then do;
mt=(1/h)##(1/alpha)#beta(1-1/alpha,1+1/alpha);
dmt=(mt#(log(h)/alpha##2+digamma(1-1/alpha)+digamma(1+1/alpha)))||(-mt/alpha/h);
vmt=dmt*vb*t(dmt); smt=sqrt(vmt); tmt=mt/smt; pmt=1-probchi(tmt##2,1);
resp=resp//(mt||smt||tmt||pmt);
tmax=((alpha-1)/h)##(1/alpha); hmax=alpha#h#tmax##(alpha-1)/(1+h#tmax##alpha);
print , "The hasard function reaches a maximum at T =" tmax,
        "Its maximum value is h =" hmax,;
           end;
		   else do;
		   print "The MEAN does not exist since alpha<=1.";
           resp=resp//(.||.||.||.);
		   end;
md=(1/h)##(1/alpha);
dmd=(md#log(h)/alpha##2)||(-md/alpha/h);
vmd=dmd*vb*t(dmd); smd=sqrt(vmd); tmd=md/smd; pmd=1-probchi(tmd##2,1);
resp=resp//(md||smd||tmd||pmd);
labc="Params"||"Std_Dev"||"Student"||"Pvalue";
labr="Log-likelihood"//"alpha"//"h"//"delta"//"m"//"sigma"//"Mean"//"Median";
print resp[colname=labc rowname=labr];
test_lom=abs(alpha-1)/sb[1]; p_lom=1-probchi(test_lom##2,1);
print "Test H0 : alpha=1 (one parameter Lomax distribution) " test_lom,
"p-value =" p_lom;
finish estim;

start prev;
tmin=0/*min(yi)*/; 
tmax=&tmax; if tmax<=0 then tmax=max(yi); tby=0.1;
do time=tmin to tmax by tby;
zi=1+h#time##alpha;
hi=alpha#h#time##(alpha-1)/zi;
si=1/zi;
fi=hi#si;
ci=1-si;
resi=resi//(time||fi||si||hi||ci);
end;
labi="&y"||"Density"||"SDF"||"Hasard"||"CDF";
finish prev;

start sortie;
create &outp from resp[colname=labc]; append from resp;
create _np from labr[colname="Estimate"]; append from labr;
create &outi from resi[colname=labi]; append from resi;
finish sortie;

run lecture;
run estim;
run prev;
run sortie;

quit;

data &outp; merge _np &outp;
run;


/* graphique 1 */
data tab1; set _outi;
keep duree_emp sdf Fonction;
Fonction="S(t)";
rename sdf=f;
run;

data tab2; set _outi;
keep duree_emp cdf Fonction;
Fonction="F(t)";
rename cdf=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;

/* graphique 2 */

data tab1; set _outi;
keep duree_emp hasard Fonction;
Fonction="h(t)";
rename hasard=f;
run;

data tab2; set _outi;
keep duree_emp density Fonction;
Fonction="f(t)";
rename density=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;


