
/* estimation des paramtres de la loi de Pareto */
libname dur "C:\Users\EDUGUET\FacOuvrages\OuvrageDuree\data";

/* VERSION OUVRAGE */


options linesize=64;

data duree; set dur.base_duree;
presence=1-censure;
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let tmax=30;
%let i=100;
%let e=1e-6;
%let c=10;
%let outp=_outp;
%let outi=_outi;


proc iml;
start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd;
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

start lv(theta) global(di,epsilon,yi,nd);
t1=theta[1]; t2=theta[2];
a=exp(t1); h=exp(t2);
lv=sum(di#t2+(di+a)#(t1-log(a+h#yi)));
return (lv);
finish;

start sc(theta) global(di,epsilon,yi,b3h);
t1=theta[1]; t2=theta[2];
a=exp(t1); h=exp(t2);
hyi=h#yi;
zi=(di+a)#hyi/(a+hyi);
der1=a#log(a/(a+hyi))+zi;
der2=di-zi;
deri=(der1||der2);
der=deri[+,];
b3h=-t(deri)*deri;
return (der);
finish;

start estim;
print "Estimation of the parameters of the Pareto distribution",,
"S(t)=(a/(a+h.t))^a",,;
epsilon=1e-6;
h0=sum(yi)/nd; a0=1;
theta0=log(a0||h0); 
lv0=lv(theta0);

/*d1=sc(theta0);
call nlpfdd(f,d2,h2,"lv",theta0);
print d1, d2; print h1, h2;
finish estim; run lecture; run estim; quit; run;*/

print "Starting parameter =" theta0,
      "Starting log-likelihood =" lv0,,;
maxc=&c; maxiter=&i; maxe=&e;
labopt="Iteration"||"Log-likelihood"||"Max Abs Elasticity"||"Sub-Iteration";
theta=theta0;
do iter=1 to maxiter;
 lambda=1;
 sc=sc(theta); 
 step=sc*inv(b3h);
 theta=theta0-lambda#step;
 lv1=lv(theta); 
 if lv1<lv0 then do;
             do c=1 to maxc;
			 lambda=lambda/2;
	         theta=theta0-lambda#step;
		     lv1=lv(theta);
             if lv1>=lv0 then c=maxc; 
			 end;
		   end;
elas=max(abs(theta#sc/lv1));
riter=(iter||lv1||elas||lambda);
print riter[colname=labopt];
theta0=theta; lv0=lv1;
if elas<&e then iter=maxiter;
end;
/*b1=exp(theta[1])||exp(theta[2]); print b1;
opt=j(11,1,0);
opt[1]=1; opt[2]=1;
h0=sum(yi)/nd; a0=1;
theta0=log(a0||h0); 
lv0=lv(theta0);
call nlpnra(rc,theta,"lv",theta0,opt);
b2=exp(theta[1])||exp(theta[2]); print b2;
finish estim; run lecture ; run estim; quit; run;*/
call nlpfdd(lv1,gr,he,"lv",theta) grd="sc";
theta=t(theta); gr=t(gr); elas=abs(theta#gr/lv1);
vp=eigval(he);
print "Log-likelihood at maximum =" lv1,
      "Gradient =" gr,
      "Elasticities =" elas,
      "Hessian eigenvalues =" vp ;
vtheta=inv(-he); 
t1=theta[1]; t2=theta[2]; b=exp(theta);
a=b[1]; h=b[2];
db=(a||0)//(0||h);
vb=db*vtheta*t(db);
sb=sqrt(vecdiag(vb));
tb=b/sb;
pb=1-probchi(tb##2,1);
resp=(b||sb||tb||pb);
if a>1 then do; mt=a/(a-1)/h;
                dmt=(-1/(a-1)##2/h)||(-mt/h);
				vmt=dmt*vb*t(dmt);
				smt=sqrt(vmt);
				tmt=mt/smt;
				pmt=1-probchi(tmt##2,1);
				resp=resp//(mt||smt||tmt||pmt);
				end;
	   else do; print "The distribution has NO mean since a<=1";
	            resp=resp//(.||.||.||.);
	            end;
md=a#(2##(1/a)-1)/h;
dmd=((2##(1/a)#(1-log(2)/a)-1)/h)||(-md/h); 
vmd=dmd*vb*t(dmd);
smd=sqrt(vmd);
tmd=md/smd;
pmd=1-probchi(tmd##2,1);
resp=(lv1||.||.||.)//resp//(md||smd||tmd||pmd);
labc="Params"||"Std_Dev"||"Student"||"Pvalue";
labr="Log-likelihood"//"a"//"h"//"Mean"//"Median";
print resp[colname=labc rowname=labr];
test_lom=abs(a-1)/sb[1]; p_lom=1-probchi(test_lom##2,1);
print "Test H0 : a=1 (one parameter Lomax distribution)" test_lom,
"p-value =" p_lom;
finish estim;

start prev;
tmin=0 /*min(yi)*/; 
tmax=&tmax; if tmax<=0 then tmax=max(yi); 
tby=0.1;
do time=tmin to tmax by tby;
zi=a/(a+h#time);
si=zi##a;
hi=h#zi;
fi=hi#si;
ci=1-si;
resi=resi//(time||fi||si||hi||ci);
end;
labi="&y"||"Density"||"SDF"||"Hasard"||"CDF";
finish prev;

start sortie;
create &outp from resp[colname=labc]; append from resp;
create _np from labr[colname="Estimate"]; append from labr;
create &outi from resi[colname=labi]; append from resi;
finish sortie;

run lecture;
run estim;
run prev;
run sortie;

quit;

data &outp; merge _np &outp;
run;



/* graphique 1 */
data tab1; set _outi;
keep duree_emp sdf Fonction;
Fonction="S(t)";
rename sdf=f;
run;

data tab2; set _outi;
keep duree_emp cdf Fonction;
Fonction="F(t)";
rename cdf=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;

/* graphique 2 */

data tab1; set _outi;
keep duree_emp hasard Fonction;
Fonction="h(t)";
rename hasard=f;
run;

data tab2; set _outi;
keep duree_emp density Fonction;
Fonction="f(t)";
rename density=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=join v=none line=3;
symbol2 c=black i=join v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;


