
/* estimation des paramtres de la loi gamma gnralise */
libname dur "C:\Users\EDUGUET\FacOuvrages\Ouvrage_Duree\data";

/* VERSION OUVRAGE */


options linesize=64;

data duree; set dur.base_duree;
presence=1-censure;
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let b_min=0.1; 
%let b_max=0.9; 
%let b_by=0.1;
/* loi Gamma  trois paramtres */
/* partie 1 : estimation par balayage du paramtre de forme (beta) */
%let outb=_outb; /* beta et lv(beta) */
%let outp=_inp; /* paramtres maximisant lv(beta) */

proc iml;
start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd; lyi=log(yi);
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

start igf(beta,x) global(epsilon);
bet=max(beta,epsilon);
f=probchi(2#x,2#bet);
do i=1 to nrow(x);
f[i]=min(f[i],1-epsilon);
end;
return (f);
finish;

start lv(theta) global(di,yi,lyi,q);
t1=theta[1]; t2=theta[2]; sigma=exp(t1); m=t2;
ui=(lyi-m)/sigma;
beta=q##(-2); psi=digamma(beta); wi=psi+q#ui;
lfi=log(q)-t1-lyi-lgamma(beta)+beta#wi-exp(wi);
lsi=log(1-igf(beta,exp(wi)));
lv=sum(di#lfi+(1-di)#lsi);
return (lv);
finish;

start estim;
print "Three-parameter Gamma distribution ",, 
"S(t)=1-g(beta,exp(digamma(beta)+q.u)",
"u=(ln(t)-m)/sigma",
"beta=q^(-2)",
"where g is the incomplete regularized Gamma function",,,
"FIRST STEP : select q and save it for the second step",,;
lyi=log(yi);
epsilon=1e-6;
m0=sum(lyi)/n; sigma0=sqrt(sum((lyi-j(n,1,m0))##2)/n);
theta0=(log(sigma0)||m0); 
opt=j(11,1,0);
opt[1]=1; opt[2]=0;
bmin=&b_min; bmax=&b_max; bby=&b_by;
nb="b"||"q"||"sigma"||"m"||"lv_q";
do b=bmin to bmax by bby;
q=b/(1-b);
call nlpnra(rc,theta,"lv",theta0,opt);
call nlpfdd(f,gr,he,"lv",theta);
elas=max(abs(theta#gr/f)); vp=max(eigval(he));
res=(b||q||exp(theta[1])||theta[2]||lv(theta));
print res[colname=nb], "Gradient =" gr, "Max Elasticity =" elas,
"Max eigenvalue =" vp ;
outb=outb//(b||q||exp(theta[1])||theta[2]||lv(theta));
theta0=theta;
end;
finish estim;

start sortie;
create &outb from outb[colname=nb]; append from outb;
call sort(outb,5,5);
outp=outb[1,];
bm=outp[1];
if bm=bmin then print "WARNING: Corner solution on the left";
if bm=bmax then print "WARNING: Corner solution on the right";
print "Maximum likelihood at " outp[colname=nb];
create &outp from outp[colname=nb]; append from outp;
finish sortie;

run lecture;
run estim;
run sortie;
quit;
run;

symbol1 c=black line=1 i=join;
proc gplot data=_outb; plot lv_q*q;
run;
