
/* estimation des paramtres de la loi de Burr */
libname dur "C:\Users\EDUGUET\FacOuvrages\Ouvrage_Duree\data";

data duree; set dur.base_duree;
presence=1-censure;
run;

/* estimation des paramtres de la loi de Burr de type XII */
/* le paramtre de forme est estim par balayage */
/* bien vrifier que la vraisemblance est maximale */


%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let b_min=0.1; 
%let b_max=0.5; 
%let b_by=0.01;
%let outb=_outb; /* beta et lv(beta) */
%let outp=_inp; /* paramtres maximisant lv(beta) */

proc iml;

start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd; lyi=log(yi);
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

/* optimisation */
/* starting value */

start lv(theta) global(di,yi,lyi,a);
t1=theta[1]; t2=theta[2]; alpha=exp(t1); h=exp(t2); 
alpha=min(alpha,100);
zi=log(1+(h/a)#yi##alpha);
lv=sum(di#(t2+t1+(alpha-1)#lyi-zi)-a#zi);
return (lv);
finish;

start sc(theta) global(di,yi,lyi,a,nd);
t1=theta[1]; t2=theta[2]; alpha=exp(t1); h=exp(t2); 
hyia=h#yi##alpha;
z2i=hyia/(a+hyia);
der1=nd+alpha#sum((di-(a+di)#z2i)#lyi);
der2=nd-sum((a+di)#z2i);
der=(der1||der2);
return (der);
finish;

start estim;
print "Burr XII distribution",, "S(t)=(a/(a+h.y^alpha))^a",,
"FIRST STEP : select a and save it for the second step",,;
alpha0=1; h0=nd/sum(yi);
theta0=log(alpha0||h0); 
lyi=log(yi);
opt=j(11,1,0);
opt[1]=1; opt[2]=0;
bmin=&b_min; bmax=&b_max; bby=&b_by;
nb="b"||"a"||"alpha"||"h"||"lv_a";
do b=bmin to bmax by bby;
 a=b/(1-b);
 call nlpnra(rc,theta,"lv",theta0,opt) grd="sc";
 call nlpfdd(f,gr,he,"lv",theta) grd="sc";
 elas=max(abs(theta#gr/f)); vp=max(eigval(he));
 res=(b||a||exp(theta[1])||exp(theta[2])||lv(theta));
 print res[colname=nb], "Gradient =" gr, "Max Elasticity =" elas,
 "Max eigenvalue =" vp ;
 outb=outb//res;
 theta0=theta;
end;
finish estim;

start sortie;
create &outb from outb[colname=nb]; append from outb;
call sort(outb,5,5);
outp=outb[1,];
bm=outp[1];
if bm=bmin then print "WARNING: Corner solution on the left";
if bm=bmax then print "WARNING: Corner solution on the right";
print "Maximum likelihood at " outp[colname=nb];
create &outp from outp[colname=nb]; append from outp;
finish sortie;

run lecture;
run estim;
run sortie;
quit;
run;

goptions htext=1.5;
symbol1 c=black line=1 i=join;
proc gplot data=_outb; plot lv_a*a/grid;
run;

