
/* estimation des paramtres du modle  hasard constant par intervalle */
libname dur "C:\Users\EDUGUET\FacOuvrages\Ouvrage_Duree\data";

/* VERSION OUVRAGE */

options linesize=80;

data duree; set dur.base_duree;
presence=1-censure;
run;


/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let outp=_outp;
%let outi=_outi;

proc freq data=&tab; tables &y*&d;
run;

proc gchart data=&tab; hbar &y/discrete;
run;

%let threshold=1 2 3 4 5 6 7 8 9 10 12 14 16 18 20 24 28 32 36 40 44 ;

proc iml;

start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd;
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

start lv(theta) global(di,yi,range,dc,c,m);
h=exp(theta);
rg1=range[1,4]:range[1,5];
Ihas=h[1]#yi[rg1];
do j=2 to m;
/* first elements of the integrated hasard */
/* y does not intervene, the intervall number does */
nj=range[j,6];
rgcj=1:(j-1);
debj=j(nj,1,1)#(t(h[rgcj])*dc[rgcj]);
/* last element of the integrated hasard */
rgrj=range[j,4]:range[j,5]; 
finj=h[j]#(yi[rgrj]-c[j-1]#j(nj,1,1));
Ihasj=debj||finj;
Ihas=Ihas//Ihasj[,+];
end;
lhi=j(range[1,6],1,log(h[1]));
do j=2 to m;
lhi=lhi//j(range[j,6],1,log(h[j]));
end;
lv=sum(di#lhi-Ihas);
return (lv);
finish;


start estim;
print "Estimation of the parameters of the piecewise constant hazard distribution",,;
c=t({&threshold}); call sort(c,1);
nc=nrow(c); m=nc+1;
dc=j(nc,1,0); dc[1]=c[1];
do j=2 to nc; dc[j]=c[j]-c[j-1]; end;
n=nrow(yi);
inter=j(n,1,0);
do i=1 to n;
if yi[i]<=c[1] then inter[i]=1;
               else do; if yi[i]>c[nc] then inter[i]=m;
			                           else do; 
									    do j=2 to nc;
									    if (c[j-1]<yi[i])&(yi[i]<=c[j]) then inter[i]=j;
									    end;
									   end;
			   end;
end;
mi=yi||di||inter;
call sort(mi,1);
yi=mi[,1]; di=mi[,2]; inter=mi[,3];
/* 1st column : intervall number */
/* 2nd column : lower bound */
/* 3rd column : upper bound */
range=j(m,7,0);
range[1,1]=1; range[1,2]=0; range[1,3]=c[1];
do j=2 to nc;
 range[j,1]=j;
 range[j,2]=range[j-1,3]; 
 range[j,3]=c[j];
end;
range[m,1]=m;
range[m,2]=c[nc];
range[m,3]=.; /* infinity */
/* 5th column : last individual index after sorting */
do i=1 to n;
   do j=1 to m;
   if inter[i]=j then range[j,5]=i;
   end;
end;
/* 4th colmun : first individual index after sorting */
range[1,4]=1;
do j=2 to m;
range[j,4]=range[j-1,5]+1;
end;
/* 6th column : number of observations */
range[,6]=range[,5]-range[,4]+j(m,1,1);
/* 7th column : percentage */
range[,7]=range[,6]/range[m,5]#100;
labrange="Intervall"||"gt_Lower"||"le_Upper"||"Block_start"||"Block_end"||"Number_Obs"||
"Percentage";
print range[colname=labrange rowname=""];
/* starting value */
h0=j(1,m,nd/sum(yi)); /* exponential case */
theta0=log(h0);
opt=j(11,1,0);
opt[1]=1; opt[2]=5;
call nlpnra(rc,theta,"lv",theta0,opt);
call nlpfdd(lv1,g,he,"lv",theta);
theta=t(theta); g=t(g); elas=abs(theta#g/lv1);
vp=eigval(he);
print "Maximum log-likelihood =" lv1,
      "Gradient =" g  "Elasticities =" elas , "Hessian eigenvalues =" vp;
vtheta=inv(-he);
h=exp(theta);
dg=diag(h);
vh=dg*vtheta*t(dg);
sh=sqrt(vecdiag(vh));
th=abs(h)/sh;
ph=1-probchi(th##2,1);
res_h=h||sh||th||pb;
labc="Hasard"||"Std_Dev"||"Student"||"Pvalue";
labr=compress(j(m,1,"h")+char(range[,1]));
print res_h[colname=labc rowname=labr];
finish estim;

start prev;
do time=0.1 to 50 by 0.01;
list=list//time;
if time<=c[1] then do; ht=ht//h[1]; iht=iht//(h[1]#time);
                       l_ht=l_ht//max(0,h[1]-1.96#sh[1]);
					   u_ht=u_ht//(h[1]+1.96#sh[1]);
              end;
              else do;
			  if time>c[nc] then do; ht=ht//h[m];
			                         l_ht=l_ht//max(0,h[m]-1.96#sh[m]);
									 u_ht=u_ht//(h[m]+1.96#sh[m]);
                                     rg=1:nc;
                                     iht=iht//(t(h[rg])*dc[rg]+h[m]#(time-c[nc]));
									 end;
			                else do;
							 do j=2 to nc;
							 if (c[j-1]<time)&(time<=c[j]) 
                                     then do;
                                     ht=ht//h[j];
									 l_ht=l_ht//max(0,h[j]-1.96#sh[j]);
									 u_ht=u_ht//(h[j]+1.96#sh[j]);
								     rg=1:(j-1);
									 iht=iht//(t(h[rg])*dc[rg]+h[j]#(time-c[j-1]));
									 end;
							 end;
						    end;
			  end;
end;
st=exp(-iht);
ct=1-st;
ft=ht#st;
labi="&y"||"Hasard"||"Density"||"SDF"||"CDF"||"l_ht"||"u_ht";
resi=list||ht||ft||st||ct||l_ht||u_ht;
/* median */
dif=abs(st-ct)||list;
call sort(dif,1);
md=dif[1,2];
print "Median duration =" md;

finish prev;

start sortie;
create &outi from resi[colname=labi]; append from resi;
create &outp from res_h[colname=labc]; append from res_h;
create _np from labr[colname="Statistic"]; append from labr;
finish sortie;

run lecture;
run estim;
run prev;
run sortie;
quit;
run;
data &outp; merge _np &outp;
run;


/* graphique 1 */
goptions htext=1.5;
symbol1 c=black i=stepjL line=1;
symbol2 c=black i=stepjL line=33;
symbol3 c=black i=stepjL line=33;
proc gplot data=_outi;
plot (hasard l_ht u_ht)*&y/overlay;
run;

data tab1; set _outi;
keep duree_emp sdf Fonction;
Fonction="S(t)";
rename sdf=f;
run;

data tab2; set _outi;
keep duree_emp cdf Fonction;
Fonction="F(t)";
rename cdf=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=stepjl v=none line=3;
symbol2 c=black i=stepjl v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;

/* graphique 2 */

data tab1; set _outi;
keep duree_emp hasard Fonction;
Fonction="h(t)";
rename hasard=f;
run;

data tab2; set _outi;
keep duree_emp density Fonction;
Fonction="f(t)";
rename density=f;
run;

data tab; set tab1 tab2;
goptions htext=1.5;
symbol1 c=black i=stepjl v=none line=3;
symbol2 c=black i=stepjl v=none line=1;
proc gplot data=tab;
plot f*duree_emp=fonction/grid;
run;

