
/* estimation des paramtres du modle  hasard constant par intervalle */
libname dur "C:\Users\EDUGUET\FacOuvrages\Ouvrage_Duree\data";

/* VERSION OUVRAGE */

options linesize=132;

data duree; set dur.base_duree;
presence=1-censure;
run;


/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let outp=_outp;
%let outi=_outi;
/* les seuils correspondent aux valeurs des dures*/
/* on les obtient par une procdure freq */
/* cette procdure donne le nombre d'observations total */
/* et censur */
proc freq noprint data=&tab; tables &y*&d/out=_total(keep=&y &d count);
proc transpose data=_total out=_total(drop=_name_ _label_) prefix=n; by &y; id &d;
data _total; set _total;
/* n0 : nbre d'obs censures, n1 : nbre d'obs compltes */
n0=max(n0,0); /* met 0 si aucune obs n'est censure */
n1=max(n1,0); /* met 0 si aucune obs n'est complte */ 
n=n0+n1; /* nombre de dures de l'intervalle */
run;
proc print;
run;


proc iml;

start lecture;
nom_y={&y};
nom_d={&d};
use &tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
nd=sum(di); n=nrow(yi); nc=n-nd;
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

start estim;
print "Kaplan-Meier estimation of the survival fonction",
"and of other associated functions (density, hasard, CDF)",,
"The mean and median duration will be estimated by assuming
that the hasard is piecewise constant over the last intervall
(when needed, for the median)";
/* programmation par bloc, on calcule le nombre d'observations total"
   et censur pour chaque valeur de la dure */
use _total; read all var {&y n0 n1 n} into mi; /* tableau des effectifs */
ny={&y};
labc=ny||"Censored"||"Complete"||"Total";
print "Data Summary",, mi[colname=labc];
n=nrow(mi); 
/* calcul des censures et des sorties cumules  partir d'une dure */
cn0=j(n,1,0); cn1=j(n,1,0); 
cn0[n]=mi[n,2]; cn1[n]=mi[n,3];
do i=n-1 to 1 by -1; /* on commence par le bas du tableau */
cn0[i]=cn0[i+1]+mi[i,2];
cn1[i]=cn1[i+1]+mi[i,3];
end;
labc=labc||"Sum_Cens"||"Sum_Comp"||"At_Risk";
mi=mi||cn0||cn1||(cn0+cn1);
print mi [colname=labc];
estim=j(n,11,0);
estim[,1]=mi[,1]; /* duration */
estim[,2]=mi[,7]; /* number at risk */
estim[,3]=mi[,3]/mi[,7]; /* hasard */
/* computation of the survival function at the observed points */
estim[1,4]=1-estim[1,3];
do i=2 to n;
estim[i,4]=(1-estim[i,3])#estim[i-1,4];
end;
/* computation of the density */
estim[1,5]=1-estim[1,4]; /* car s(0)=1 */
do i=2 to n;
estim[i,5]=estim[i-1,4]-estim[i,4];
end;
estim[,6]=1-estim[,4];
estim[,7]=1-estim[,3];
estim[,8]=sqrt(estim[,7]#(1-estim[,7])/estim[,2]); /* std hasard = std pi */
/* cart-type de la survie */
estim[1,9]=(1-estim[1,7])/(estim[1,2]#estim[1,7]);
do i=2 to n;
estim[i,9]=estim[i-1,9]+(1-estim[i,7])/(estim[i,2]#estim[i,7]);
end;
estim[,10]=estim[,4]#sqrt(estim[,9]);
/* cart-type de la densit */
estim[1,11]=estim[1,10];
do i=2 to n;
sj1=estim[i-1,4];
sj=estim[i,4];
vj1=estim[i-1,9];
vj=estim[i,9];
estim[i,11]=sqrt(max(0,sj1##2#vj1+sj##2#vj-2#sj#sj1#vj1));
end;
/* ligne en 0 pour la survie et la f.r. seulement */
ligne0=(0||.||.||1||.||0||.||.||.||.||.);
estim=ligne0//estim;
labec=ny||"At_Risk"||"Hasard"||"SDF"||"Density"||"CDF"||"Pi"||"Std_has"||"Calc"||"Std_SDF"||"Std_pdf";
create _tab from estim[colname=labec]; append from estim;
print estim[colname=labec];
finish estim;


start sortie;
create &outi from resi[colname=labi]; append from resi;
create &outp from res_h[colname=labc]; append from res_h;
create _np from labr[colname="Statistic"]; append from labr;
finish sortie;

run lecture;
run estim;
/*run prev;
run sortie;*/
quit;
run;

symbol1 c=black line=1 interpol=stepJR;
proc gplot data=_tab;
plot sdf*&y;
run;

symbol1 c=black line=1 interpol=stepJR;
proc gplot data=_tab;
plot cdf*&y;
run;

symbol1 c=black line=1 interpol=stepJR;
proc gplot data=_tab;
plot hasard*&y;
run;

symbol1 c=black line=1 interpol=stepJR;
proc gplot data=_tab;
plot density*&y;
run;
