
/* Estimation non paramtrique */
/* avec calcul des log vraisemblance */

libname dur "C:\Users\EDUGUET\FacOuvrages\OuvrageDuree\data";
options linesize=80;

data duree; set dur.base_duree; presence=1-censure;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let d=presence; /* indicatrice de dure complte */
%let outkm=_Kaplan_Meier;
%let outb=_Breslow;
%let outs=_stats;

/* les seuils correspondent aux valeurs des dures*/
/* on les obtient par une procdure freq */
/* cette procdure donne le nombre d'observations total */
/* et censur */

proc freq noprint data=&tab; tables &y*&d/out=_total(keep=&y &d count);
proc transpose data=_total out=_total(drop=_name_ _label_) prefix=n; by &y; id &d;
data _total; set _total;
/* n0 : nbre d'obs censures, n1 : nbre d'obs compltes */
n0=max(n0,0); /* met 0 si aucune obs n'est censure */
n1=max(n1,0); /* met 0 si aucune obs n'est complte */ 
ntot=n0+n1; /* nombre de dures de l'intervalle */
run;


proc iml;

start lecture;
nom_y={&y};
use _total;
    read all var {n0} into ncens;
	read all var {n1} into ncomp;
	read all var {ntot} into ntot;
	read all var (nom_y) into dur;
tab=dur||ncens||ncomp||ntot;
labtab=nom_y||"CENSORED"||"FAILED"||"TOTAL";
print "DATA SUMMARY",
      "************",,, tab[colname=labtab];
finish lecture;

start estim;
ndur=nrow(tab);
c_ncens=j(ndur,1,0); c_ncomp=j(ndur,1,0); 
c_ncens[ndur]=ncens[ndur]; c_ncomp[ndur]=ncomp[ndur];
do j=ndur-1 to 1 by -1;
c_ncens[j]=c_ncens[j+1]+ncens[j];
c_ncomp[j]=c_ncomp[j+1]+ncomp[j];
end;
c_AtRisk=c_ncens+c_ncomp;
tab2=dur||c_ncens||c_ncomp||c_AtRisk;
labtab2=nom_y||"Cum_Censored"||"Cum_Failure"||"At_Risk";
print "Population At Risk",
      "******************",,, tab2 [colname=labtab2];
/* estimation du hasard */
haz=ncomp/c_AtRisk;
/* Coefficient pi */
pi=1-haz;
/* Survie Kaplan Meier */
sdf_km=j(ndur,1,0);
sdf_km[1]=pi[1];
do j=2 to ndur;
sdf_km[j]=pi[j]#sdf_km[j-1];
end;
/* Variances de pi, ln(pi), ln(sdf_km), et sdf_km */
vas_pi=pi#(1-pi)/c_AtRisk;
vas_ln_pi=(1-pi)/(pi#c_AtRisk);
vas_ln_sdf_km=j(ndur,1,0);
vas_ln_sdf_km[1]=vas_ln_pi[1];
do j=2 to ndur;
vas_ln_sdf_km[j]=vas_ln_sdf_km[j-1]+vas_ln_pi[j];
end;
vas_sdf_km=(sdf_km##2)#vas_ln_sdf_km;
/* Hasard cumul */
Aalen=j(ndur,1,0);
Aalen[1]=haz[1];
do j=2 to ndur;
Aalen[j]=Aalen[j-1]+haz[j];
end;
/* Variance Aalen */
vas_aa=j(ndur,1,0);
vas_aa[1]=vas_pi[1];
do j=2 to ndur;
vas_aa[j]=vas_aa[j-1]+vas_pi[j];
end;
/* Survie Breslow */
sdf_b=exp(-Aalen);
/* variance survie Breslow */
vas_sdf_b=(sdf_b##2)#vas_ln_sdf_km; /* ce n'est pas une erreur */
/* CDF Kaplan Meier et Breslow */
cdf_km=1-sdf_km;
cdf_b=1-sdf_b;
/* PDF Kaplan Meier et Breslow */
pdf_km=j(ndur,1,0); pdf_b=j(ndur,1,0);
pdf_km[1]=1-sdf_km[1]; pdf_b[1]=1-sdf_b[1];
do j=2 to ndur;
pdf_km[j]=sdf_km[j-1]-sdf_km[j];
pdf_b[j]=sdf_b[j-1]-sdf_b[j];
end;
/* Variance pdf */
vas_pdf_km=j(ndur,1,0); vas_pdf_b=j(ndur,1,0);
vas_pdf_km[1]=vas_sdf_km[1]; /* car f(1)=s(0)-s(1) et s(0)=1*/
vas_pdf_b[1]=vas_sdf_b[1];
do j=2 to ndur;
vas_pdf_km[j]=vas_sdf_km[j-1]+vas_sdf_km[j]-2#sdf_km[j-1]#sdf_km[j]#vas_ln_sdf_km[j-1];
vas_pdf_b[j]=vas_sdf_b[j-1]+vas_sdf_b[j]-2#sdf_b[j-1]#sdf_b[j]#vas_ln_sdf_km[j-1]; /* ce n'est pas une erreur */
end;

std_sdf_km=sqrt(vas_sdf_km);
std_haz=sqrt(vas_pi);
std_pdf_km=sqrt(vas_pdf_km);

mat_km_1=sdf_km||std_sdf_km||cdf_km;
mat_km_2=haz||std_haz||pdf_km||std_pdf_km;
char_dur=char(dur);


labkm1="sdf_km"||"std_sdf_km"||"cdf_km";
labkm2="hasard"||"std_hasard"||"pdf_km"||"std_pdf_km";
print "Kaplan Meier estimates",, mat_km_1[colname=labkm1 rowname=char_dur],
mat_km_2[colname=labkm2 rowname=char_dur];

std_sdf_b=sqrt(vas_sdf_b);
std_pdf_b=sqrt(vas_pdf_b);

mat_b_1=sdf_b||std_sdf_b||cdf_b;
mat_b_2=haz||std_haz||pdf_b||std_pdf_b;

labb1="sdf_Breslow"||"std_sdf_Breslow"||"cdf_Breslow";
labb2="hasard"||"std_hasard"||"pdf_Breslow"||"std_pdf_Breslow";
print "Breslow estimates",, mat_b_1[colname=labb1 rowname=char_dur],
mat_b_2[colname=labb2 rowname=char_dur];

/* log-vraisemblances */
/* on enlve les densits nulles */
do j=1 to nrow(pdf_km);
if pdf_km[j]>0 then lv_km=lv_km//(ncomp[j]#log(pdf_km[j])+ncens[j]#log(sdf_km[j]));
if pdf_b[j]>0  then  lv_b=lv_b //(ncomp[j]#log(pdf_b[j])+ncens[j]#log(sdf_b[j]));
end;
lv_km=sum(lv_km); lv_b=sum(lv_b);
print "Log-likelihood Kaplan-Meier (Best) =" lv_km,
      "Log-likelihood Breslow =" lv_b;
finish estim;

start sortie ;
outkm=dur||mat_km_1||mat_km_2;
labckm=nom_y||labkm1||labkm2;
create &outkm from outkm[colname=labckm]; append from outkm;

outb=dur||mat_b_1||mat_b_2;
labcb=nom_y||labb1||labb2;
create &outb from outb[colname=labcb]; append from outb;

tab=tab||c_atrisk; labtab=labtab||"AT_RISK";
create &outs from tab[colname=labtab]; append from tab;
finish sortie;

run lecture;
run estim;
run sortie;
quit;
run;

data tab1; set _kaplan_meier(keep=&y cdf_km);
rename cdf_km=f;
function="CDF";

data tab2; set _kaplan_meier(keep=&y sdf_km);
rename sdf_km=f;
function="SDF";

data tab; set tab1 tab2;
run;

goptions htext=1.5;
symbol1 c=black line=1 interpol=stepJR;
symbol2 c=black line=3 interpol=stepJR;

proc gplot data=tab;
plot f*&y=function;
run;

proc gplot data=tab;
plot f*&y=function/haxis=0 to 30 by 1;
run;


data tab1; set _breslow(keep=&y sdf_breslow);
length function $15.;
rename sdf_breslow=f;
function="Breslow";

data tab2; set _kaplan_meier(keep=&y sdf_km);
length function $15.;
rename sdf_km=f;
function="Kaplan-Meier";

data tab; set tab1 tab2;
run;

goptions htext=1.5;
symbol1 c=black line=1 interpol=stepJR;
symbol2 c=black line=3 interpol=stepJR;

proc gplot data=tab;
plot f*&y=function;
run;

proc gplot data=tab;
plot f*&y=function/haxis=0 to 30 by 1;
run;

proc gplot data=_kaplan_meier;
plot hasard*&y;
run;

data tab; set _kaplan_meier; if &y<=45;
proc gplot data=tab;
plot hasard*&y;
run;



data tab; set _kaplan_meier; 
proc gplot data=tab;
plot pdf_km*&y/haxis=0 to 55 by 5;
run;
