libname dur "C:\Users\EDUGUET\FacOuvrages\OuvrageDuree\data";

/* VERSION OUVRAGE */


/* estimation des paramtres de la loi de Weibull */

options linesize=64;

data duree; set dur.base_duree;
presence=1-censure;
c_nais1=(anaisenq<=1948);
c_nais2=(1948<anaisenq<=1958);
c_nais3=(1958<anaisenq<=1968);
c_nais4=(1968<anaisenq<=1978);
c_nais5=(1978<anaisenq<=1988);
parents=(fpermer=1);
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let x=/*dip_pri*/ dip_sec dip_sup /*gen_h*/  gen_f/*france d_metr d_petr*/ /*c_nais2 c_nais3 c_nais4*/
c_nais2 c_nais3 c_nais4 c_nais5;
%let d=presence; /* indicatrice de dure complte */
%let outp=_outp;
%let outi=_outi;
%let e=1e-6;
%let maxiter=300;
%let maxc=10;
/* centrage des variables */
proc means; var &y &x; run;
proc standard data=&tab out=_tab mean=0; var &x; run;
data _tab; set _tab; nais2=anaisenq**2;

proc iml;
start lecture;
nom_y={&y};
nom_d={&d};
/**/ nom_x={&x};
use _tab; read all var(nom_y) into yi;
          read all var(nom_d) into di;
		  read all var(nom_x) into xi;
nd=sum(di); n=nrow(yi); nc=n-nd;
/**/ xi=j(n,1,1)||xi; nx="Intercept"||nom_x;
p=ncol(xi); rgb=1:p; rga=p+1;
lyi=log(yi);
print "Time variable =" nom_y,
      "Censoring variable (=0 if censored) =" nom_d,
	  "Number of observations =" n,
	  "Complete =" nd,
	  "Censored =" nc;
finish lecture;

/* log-likelihood */
start lv(theta) global(di,lyi,xi,rgb,rga);
b=theta[rgb];
alpha=exp(theta[rga]);
alyi=alpha#lyi;
mi=xi*t(b);
lv=sum(di#(mi+ayi-lyi+log(alpha))-exp(mi+alyi));
return (lv);
finish;


start estim;
print "Weibull distribution",, "S(t)=exp(-h.t^alpha)",,;
theta0=t(ginv(xi)*(-log(yi))||log(1);
opt=j(11,1,0); opt[1]=1; opt[2]=5;
call nlpnra(rc,theta,"lv",theta0,opt);
call nlpfdd(lv1,gr,he,"lv",theta);
theta=t(theta); gr=t(gr); vp=eigval(he);
elas1=abs(theta#gr/lv1);
lab1="Gradient"||"Elasticities"||"Hessian_Eigenvalues";
out1=gr||elas1||vp;
print out1[colname=lab1];
parm=theta[rgp]//exp(theta[rga]);
dg=(i(p)||j(p,1,0))//(j(p,1,0)||parm[p+1]);
vtheta=inv(-he);
vparm=dg*vtheta*t(dg);
sb1=sqrt(vecdiag(vtheta));
tb1=abs(theta)/sb1;
pb1=1-probchi(tb1##2,1);
lab2="Coeff"||"Std_Dev"||"Student"||"P_value";
out2=theta||sb1||tb1||pb1;
print out2[rowname=nx colname=nx];
hm=exp(theta[1]); shm=hm#sb1[1];
hm_L=max(0,hm-1.96#shm); hm_U=hm+1.96#shm;
print "Hasard at the average point of the sample :"  hm,
"95% confidence interval :" hm_L "," hm_U;
p=ncol(xi); rg=2:p; 
b=theta[rg]; vb=vtheta[rg,rg];
Wald=t(b)*inv(vb)*b;
pWald=1-probchi(Wald,p);
print "Wald test of global significance", ,
      "Statistic =" Wald,
	  "p value =" pWald;
finish estim;

start prev;
hi=exp(xi*theta);
create hi from hi[colname="hasard"]; append from hi;
finish prev;

run lecture;
run estim;
run prev;
quit;
run;

proc means; var hasard;
run;

proc kde data=hi; univar hasard  /out=densite;
run;

goptions htext=2;
symbol1 c=black i=join line=1;
proc gplot data=densite;
plot density*value=var/grid;
run;
