libname dur "C:\Users\EDUGUET\FacOuvrages\OuvrageDuree\data";

/* VERSION OUVRAGE */


/* estimation des paramtres de la loi de Weibull */

options linesize=64;

data duree; set dur.base_duree;
presence=1-censure;
c_nais1=(anaisenq<=1948);
c_nais2=(1948<anaisenq<=1958);
c_nais3=(1958<anaisenq<=1968);
c_nais4=(1968<anaisenq<=1978);
c_nais5=(1978<anaisenq<=1988);
run;

/* paramtres du programme */
%let tab=duree; /* nom du tableau SAS contenant les donnes */
%let y=duree_emp; /* variable de dure */
%let x=/*dip_pri*/ dip_sec dip_sup /*gen_h*/  gen_f/*france d_metr d_petr*/ /*c_nais2 c_nais3 c_nais4*/
c_nais2 c_nais3 c_nais4 c_nais5;
%let d=presence; /* indicatrice de dure complte */
/* centrage des variables */

proc standard data=&tab out=_tab mean=0; var &x; run;

proc lifereg data=_tab outest=b covout;
model &y*&d(0)=&x;
output out=_sortie xbeta=xbeta;
run;

data b; set b; if compress(upcase(_name_))="SCALE" then _name_="_SCALE_";
run;
/* Tableau d'estimations */

data parms; set b; if _type_="PARMS";
proc print;
data cov; set b; if _type_="COV";
proc print;
run;

proc iml;
use cov; read all var{_name_} into nx; nx=compress(nx);
         read all var(nx) into vtheta;
use parms; read all var (nx) into theta;
theta=t(theta); p=nrow(theta);
 rgb=1:(p-1); rgs=p; 
beta=theta[rgb]; sigma=theta[rgs];
b=(1/sigma)//(-beta/sigma); /* chang */
nb="Alpha"//nx[rgb];
dg=(    j(1,p-1,0)   || (-1/sigma##2) )//
   ( (-i(p-1)/sigma) || (beta/sigma##2) )
   ;
vb=dg*vtheta*t(dg);
sb=sqrt(vecdiag(vb));
tb=abs(b)/sb;
pb=1-probchi(tb##2,1);
print "Coefficients for the hasard function",
      "with parametrisation ",
	  "h0(t)=alpha.h.t^(alpha-1)", ,
      b[rowname=nb] sb tb pb;
stat=b[1]/sb[1];
pstat=1-probchi(stat##2,1);
print "Test H0: exponential distribution",
       "Statistic =" stat,
	   "p value =" pstat;
/* Distribution de base */
alpha=b[1]; h=exp(b[2]); 
do t=0 to 1 by 0.01;
valt=valt//t;
hbase=hbase//(alpha#h#t##(alpha-1));
sbase=sbase//(exp(-h#t##alpha));
end;
do t=1 to 30 by 0.1;
valt=valt//t;
hbase=hbase//(alpha#h#t##(alpha-1));
sbase=sbase//(exp(-h#t##alpha));
end;
fbase=hbase#sbase;
cbase=j(nrow(sbase),1,1)-sbase;
base=valt||hbase||fbase||sbase||cbase;
labase="Duration"||"Hasard"||"PDF"||"SDF"||"CDF";
create base from base[colname=labase]; append from base;
/* Prvision de la fonction de survie */
rgc=2:(p-1); nxc=nx[rgc]; rg1=2:p; 
use _sortie; read all var(nxc) into xc;
n=nrow(xc); 
xt=j(n,1,1)||xc;
do t=1 to 5;
s0=s0||exp(-t##alpha)##exp(xt*b[rg1]);
ns0=ns0||compress("S"+char(t));
end;
create prev from s0[colname=ns0]; append from s0;
/* estimations de la dure moyenne */
et0=gamma(1+1/alpha)/(h##(1/alpha));
print "Dure moyenne au point moyen", et0;
rg2=3:p;
eti=et0/exp(xc*b[rg2]/alpha);
create eti from eti[colname="Mean"]; append from eti;
quit;
run;
proc print data=prev(obs=20);
run;
proc univariate plot data=eti; var mean;
run;

goptions htext=2;
symbol1 c=black line=1 v=none i=join;
symbol2 c=black line=3 v=none i=join;

proc gplot data=base;
plot (hasard pdf)*duration/overlay grid;
run;
proc gplot data=base;
plot (SDF CDF)*duration/overlay grid;
run;

proc kde data=prev; univar s5/out=noyau;
run;
proc gplot data=noyau; plot density*value=var/grid;run;

proc kde data=eti; univar Mean/out=noyau;
run;
proc gplot data=noyau; plot density*value=var/grid;
run;

proc means data=duree q1 median q3 mean; var &y; class censure;
run;

proc print data=b;
run;
